function AllInOneInventory_CreateButtons(frame)
	frame.bags = {};
	local bagName = nil;
	-- Kludge to increase maximum number of slots.
	local maxBagSlots = 30;
	local totalSlots = 0;
	for bag = 0, 4 do
		local slots = GetContainerNumSlots(bag);
		totalSlots = totalSlots + slots;
		if ( slots ) and ( slots > maxBagSlots ) then
			maxBagSlots = slots;
		end
	end
	local slotName = nil;
	for i = 1, totalSlots do
		slotName = frame:GetName().."ItemTexture"..i;
		if ( not getglobal(slotName) ) then
			frame:CreateTexture(slotName, "ARTWORK", "AllInOneInventoryFrameTextureTemplate");
		end
	end
	for bag = 0, 4 do
		bagName = frame:GetName().."Bag"..bag;
		if not frame.bags[bag] then 
			frame.bags[bag] = CreateFrame("Frame", bagName, frame)
			frame.bags[bag]:SetID(bag);
		end
		frame.bags[bag].size = GetContainerNumSlots(bag);
		for slot = 1, maxBagSlots do
			if not frame.bags[bag][slot] then
				frame.bags[bag][slot] = CreateFrame("Button", bagName.."Item"..slot, frame.bags[bag], "ContainerFrameItemButtonTemplate")
				frame.bags[bag][slot]:SetID(slot);
			end
		end
	end
end

function AllInOneInventory_UpdateBags(frame)
	local bag = 0;
	for bag = 0, 4 do 
		AllInOneInventory_UpdateBag(frame, bag);
	end
end

function AllInOneInventory_UpdateBag(frame, bag)
	if ( not bag ) then return false; end
	if ( not frame.bags ) then return false; end
	if ( frame.bags[bag] ) then
		ContainerFrame_Update(frame.bags[bag]);

		local bagBase = frame:GetName().."Bag";
		local bagName = bagBase..bag;
		local baseSlotName = bagName.."Item";
		local itemButton = nil;
		
		for slot = 1, GetContainerNumSlots(bag) do
			itemButton = getglobal(baseSlotName..slot);
			if ( itemButton ) then
				local texture, itemCount, locked, quality, readable = GetContainerItemInfo(bag, slot);
				itemButton.readable = readable;
				SetItemButtonCount(itemButton, itemCount);
				SetItemButtonDesaturated(itemButton, locked, 0.5, 0.5, 0.5);
				if ((quality) and (quality < (AllInOneInventory_Quality_Filter - 1))) then
					if ( AllInOneInventory_Filter_Debug ) then
						AllInOneInventory_Print("quality match: " .. itemInfo.n .. " : ".. itemInfo.q .. ":".. AllInOneInventory_Item_Qualities[AllInOneInventory_Quality_Filter]);
					end
					if ( ( texture ) and ( strlen(texture) > 0 ) ) then
						if ( AllInOneInventory_Filter_Debug ) then
							AllInOneInventory_Print("current texture: " .. itemInfo.t);
						end
						SetItemButtonTexture(itemButton, AllInOneInventory_FilteredTexture);
					else
						SetItemButtonTexture(itemButton, texture);
					end
				else
					SetItemButtonTexture(itemButton, texture);
				end
			end
		end
		return true;
	end
end


function AllInOneInventory_OrganizeButtons(frame, columns)
	local bag = 0;
	local slot = 1;
	local totalSlot = 0;
	local bagBase = frame:GetName().."Bag";
	local bagName = bagBase..bag;
	local slotSuffix = "Item";
	local itemButton = nil;	
	local previousRow = nil;
	local previous = nil;

	local bagSlots = 0;
	for bag = 0, 4 do
		bagName = bagBase..bag;
		bagSlots = GetContainerNumSlots(bag);
		useSlots = AllInOneInventory_GetNumberOfSlotsInBag(bag);
		for slot = 1, useSlots do
			itemButton = getglobal(bagName..slotSuffix..slot);

			totalSlot = totalSlot + 1;
			itemButton:ClearAllPoints();
			itemButton.slot = totalSlot;
			if ( totalSlot <= 1 ) then
				-- Set first button upper left corner.
				itemButton:SetPoint("TOPLEFT", frame:GetName().."ItemTexture1", "TOPLEFT", 4, -1);
				previousRow = itemButton;
			elseif ( mod(totalSlot, columns) == 0 ) then
				itemButton:SetPoint("TOPLEFT", previousRow:GetName(), "TOPLEFT", 0, 41);
				previousRow = itemButton;
			else
				itemButton:SetPoint("TOPLEFT", previous:GetName(), "TOPLEFT", -42, 0);
			end
			previous = itemButton;
			itemButton:Show();
		end
		for k, v in pairs(frame.bags[bag]) do
			if ( type(k) == "number" ) and ( k > useSlots+1 ) then
				v:Hide();
				v.slot = nil;
			end
		end
	end
end



function AllInOneInventoryItemsFrame_OnLoad()
	local f = AllInOneInventoryItemsFrame;
	f:RegisterEvent("VARIABLES_LOADED");
	f:RegisterEvent("UPDATE_INVENTORY_ALERTS");
	f:RegisterEvent("BAG_UPDATE");
	f:RegisterEvent("BAG_UPDATE_COOLDOWN");
end

function AllInOneInventoryItemsFrame_OnEvent(event)
	local f = AllInOneInventoryItemsFrame;

	if ( event == "VARIABLES_LOADED" ) then
		f:UnregisterEvent(event);
	elseif (event == "UPDATE_INVENTORY_ALERTS" ) then
		AllInOneInventory_UpdateBags(f);
	else
		local n = tonumber(arg1);
		if ( n ) then
			AllInOneInventory_UpdateBag(arg1);
		end
	end
end

